/*
 * Copyright (C) 2009 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145 NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef FIELDBUS_H
#define FIELDBUS_H

#if defined(__linux__) || defined(__unix__)

#ifndef _POSIX_C_SOURCE
#define _POSIX_C_SOURCE 199309L
#endif

#endif

#include <stdlib.h>

#include "connection.h"

/* Definition of the FieldBus incoming packets, includes a timestamp and async values */
typedef struct FieldBusData
{
    double dTimestamp;
    float *fValues;
} FieldBusData;

/* Definition of the FieldBus Ring Buffer */
typedef struct FieldBusRingBuf
{
    FieldBusData* buffer;
    size_t writeIndex;
    size_t readIndex;
    size_t capacity;
    int overrun;
    int dataCount;
} FieldBusRingBuf;

/* Definition of the FieldBus Ring Buffer Handle */
typedef FieldBusRingBuf *FieldBusRingBufHandle;

/* Struct Definition for the FieldBus Connection*/
typedef struct FieldBus
{
    FieldBusRingBufHandle dataHandle;
    GHSConnectionHandle conHandle;
    char *ipAddress;
    FieldBusData LastPacket;
} FieldBus;

/* Definition of the FieldBus Handle */
typedef FieldBus* FieldBusHandle;

/* Initialization of the FieldBus Ring Buffer API */
FieldBusRingBufHandle fieldBus_buf_Init(size_t size, int dataCount);

/* Retrieves the FieldBus Ring Buffer size (difference between writing and reading index) */
size_t fieldBus_buf_GetSize(FieldBusRingBufHandle handle);

/* Destruction of the FieldBus Ring Buffer API (frees the structure memory and its value array) */
void fieldBus_buf_Free(FieldBusRingBufHandle handle);

/* Insert FieldBusData in the FieldBus Ring Buffer */
void fieldBus_buf_PutAsyncData(FieldBusRingBufHandle rbuf, double timestamp, float *values);

/* Read FieldBusData in the FieldBus Ring Buffer */
int fieldBus_buf_GetAsyncData(FieldBusRingBufHandle rbuf, double *timestamp, float *values, GHSFunctionCallType callType);

/* Read FieldBusData packet from the FieldBus socket connection */
GHSReturnValue FieldBusConnectionRead(GHSConnectionHandle *fieldBusHandle, FieldBusData *packet, int dataCount);

#endif /* FIELDBUS_H */
