/*
 * Copyright (C) 2009 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145 NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "socket.h"
#include "errno.h"

int SockWrite(socket_t sock, const char *buf, int count)
{
#if defined(_WIN32)
    return send(sock, buf, count, 0);
#elif defined(__linux__) || defined(__unix__)
    int nBytes = 0;
    do
    {
        nBytes = write(sock, buf, count);
    } while (nBytes == -1 && errno == EINTR);
    /* any other error: connection is closed */
    return nBytes;
#else
#error Unsupported platform
#endif
}

int SockRead(socket_t sock, char *buf, int count)
{
#if defined(_WIN32)
    return recv(sock, buf, count, 0);
#elif defined(__linux__) || defined(__unix__)
    int nBytes = 0;
    do
    {
        nBytes = read(sock, buf, count);
    } while (nBytes == -1 && errno == EINTR);
    /* any other error: connection is closed */
    return nBytes;
#else
#error Unsupported platform
#endif
}

int SockClose(socket_t sock)
{
#if defined(_WIN32)
    return closesocket(sock);
#elif defined(__linux__) || defined(__unix__)
    return close(sock);
#else
#error Unsupported platform
#endif
}

/* SockInit: Initialize Socket library */
/* Return:  0 on success */
/* Each successful call to SockInit must be matched with a call to SockClose */
int SockInit(void)
{
#if defined(_WIN32)
    WSADATA wsaData;
    if (WSAStartup(MAKEWORD(2, 2), &wsaData) != 0)
        return -1;
#elif defined(__linux__) || defined(__unix__)
#else
#error Unsupported platform
#endif
    return 0;
}

void SockCleanup(void)
{
#if defined(_WIN32)
    WSACleanup();
#elif defined(__linux__) || defined(__unix__)
#else
#error Unsupported platform
#endif
}
